@@ -26,8 +26,17 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk-pixbuf/gdk-pixbuf-io.h>
 
+#if HAVE_SVGZ
+#include <rsvg-gz.h>
+#endif
+
 typedef struct {
         RsvgHandle                 *handle;
+
+#if HAVE_SVGZ
+        gboolean                    first_write;
+#endif
+
         GdkPixbuf                  *pixbuf;
         GdkPixbufModuleUpdatedFunc  updated_func;
         GdkPixbufModulePreparedFunc prepared_func;
@@ -46,7 +55,14 @@ gdk_pixbuf__svg_image_begin_load (GdkPixbufModuleSizeFunc size_func,
 {
         SvgContext *context    = g_new0 (SvgContext, 1);
 
+#if HAVE_SVGZ
+        /* lazy create the handle on the first write */
+        context->handle        = NULL;
+        context->first_write   = TRUE;
+#else
         context->handle        = rsvg_handle_new ();
+#endif
+
         context->prepared_func = prepared_func;
         context->updated_func  = updated_func;
         context->user_data     = user_data;
@@ -62,7 +78,21 @@ gdk_pixbuf__svg_image_load_increment (gpointer data,
 				      GError **error)
 {
         SvgContext *context = (SvgContext *)data;
-        gboolean result     = rsvg_handle_write (context->handle, buf, size, error);  
+        gboolean result;
+
+#if HAVE_SVGZ
+        if (context->first_write == TRUE) {
+                context->first_write = FALSE;
+
+                /* lazy create a SVGZ or SVG loader */
+                if ((size >= 2) && (buf[0] == (guchar)0x1f) && (buf[1] == (guchar)0x8b))
+                        context->handle = rsvg_handle_new_gz ();
+                else
+                        context->handle = rsvg_handle_new ();
+        }
+#endif
+
+        result = rsvg_handle_write (context->handle, buf, size, error);  
 
         context->pixbuf = rsvg_handle_get_pixbuf (context->handle);
   
@@ -115,6 +145,9 @@ fill_info (GdkPixbufFormat *info)
                 { "<?xml", NULL, 50 },
                 { "<svg", NULL, 100 },
                 { "<!DOCTYPE svg", NULL, 100 },
+#if HAVE_SVGZ
+                { "\x1f\x8b", NULL, 50 }, /* todo: recognizes any gzipped file, not much we can do */
+#endif
                 { NULL, NULL, 0 }
         };
         static gchar *mime_types[] = { 
@@ -124,6 +157,9 @@ fill_info (GdkPixbufFormat *info)
         };
         static gchar *extensions[] = { 
                 "svg", 
+#if HAVE_SVGZ
+                "svgz",
+#endif
                 NULL 
         };
         
