@@ -22,6 +22,7 @@
 
    Author: Raph Levien <raph@artofcode.com>
 */
+#include <string.h>
 
 #include "rsvg-shapes.h"
 #include "rsvg-styles.h"
@@ -265,8 +266,11 @@ rsvg_start_path (RsvgHandle *ctx, const xmlChar **atts)
 {
 	int i;
 	char *d = NULL;
-	const char * klazz = NULL;
+	const char * klazz = NULL, * id = NULL;
 	
+	/* skip over defs entries for now */
+	if (ctx->in_defs) return;
+
 	if (atts != NULL)
 		{
 			for (i = 0; atts[i] != NULL; i += 2)
@@ -275,13 +279,15 @@ rsvg_start_path (RsvgHandle *ctx, const xmlChar **atts)
 						d = (char *)atts[i + 1];
 					else if (!strcmp ((char *)atts[i], "class"))
 						klazz = (char *)atts[i + 1];
+					else if (!strcmp ((char *)atts[i], "id"))
+						id = (const char *)atts[i + 1];
 				}
 		}
 	
 	if (d == NULL)
 		return;
 	
-	rsvg_parse_style_attrs (ctx, "path", klazz, atts);
+	rsvg_parse_style_attrs (ctx, "path", klazz, id, atts);
 	rsvg_render_path (ctx, d);
 }
 
@@ -313,14 +319,17 @@ rsvg_make_poly_point_list(const char * points)
 static void
 rsvg_start_any_poly(RsvgHandle *ctx, const xmlChar **atts, gboolean is_polyline)
 {
-	/* the only difference i'm making between polygon and polyline is
+	/* the only difference between polygon and polyline is
 	   that a polyline closes the path */
 	
 	int i;
 	const char * verts = (const char *)NULL;
 	GString * g = NULL;
 	gchar ** pointlist = NULL;
-	const char * klazz = NULL;
+	const char * klazz = NULL, * id = NULL;
+
+	/* skip over defs entries for now */
+	if (ctx->in_defs) return;
 	
 	if (atts != NULL)
 		{
@@ -331,13 +340,15 @@ rsvg_start_any_poly(RsvgHandle *ctx, const xmlChar **atts, gboolean is_polyline)
 						verts = (const char *)atts[i + 1];
 					else if (!strcmp ((char *)atts[i], "class"))
 						klazz = (const char *)atts[i + 1];
+					else if (!strcmp ((char *)atts[i], "id"))
+						id = (const char *)atts[i + 1];
 				}
 		}
 	
 	if (!verts)
 		return;
 	
-	rsvg_parse_style_attrs (ctx, (is_polyline ? "polyline" : "polygon"), klazz, atts);
+	rsvg_parse_style_attrs (ctx, (is_polyline ? "polyline" : "polygon"), klazz, id, atts);
 	
 	/* todo: make the following more memory and CPU friendly */
 	g = rsvg_make_poly_point_list (verts);
@@ -380,10 +391,15 @@ rsvg_start_line (RsvgHandle *ctx, const xmlChar **atts)
 	int i;
 	double x1 = 0, y1 = 0, x2 = 0, y2 = 0;
 	GString * d = NULL;
-	const char * klazz = NULL;
-	RsvgState *state = &ctx->state[ctx->n_state - 1];
+	const char * klazz = NULL, * id = NULL;
+	RsvgState *state;
 	char buf [G_ASCII_DTOSTR_BUF_SIZE];
 
+	/* skip over defs entries for now */
+	if (ctx->in_defs) return;
+
+	state = &ctx->state[ctx->n_state - 1];
+
 	if (atts != NULL)
 		{
 			for (i = 0; atts[i] != NULL; i += 2)
@@ -398,9 +414,11 @@ rsvg_start_line (RsvgHandle *ctx, const xmlChar **atts)
 						y2 = rsvg_css_parse_normalized_length ((char *)atts[i + 1], ctx->dpi, (gdouble)ctx->height, state->font_size);
 					else if (!strcmp ((char *)atts[i], "class"))
 						klazz = (const char *)atts[i + 1];
+					else if (!strcmp ((char *)atts[i], "id"))
+						id = (const char *)atts[i + 1];
 				}      
 		}
-	rsvg_parse_style_attrs (ctx, "line", klazz, atts);
+	rsvg_parse_style_attrs (ctx, "line", klazz, id, atts);
 	
 	/* emulate a line using a path */
 	/* ("M %f %f L %f %f", x1, y1, x2, y2) */
@@ -424,11 +442,16 @@ rsvg_start_rect (RsvgHandle *ctx, const xmlChar **atts)
 	int i;
 	double x = -1, y = -1, w = -1, h = -1, rx = 0., ry = 0.;
 	GString * d = NULL;
-	const char * klazz = NULL;
-	RsvgState *state = &ctx->state[ctx->n_state - 1];
+	const char * klazz = NULL, * id = NULL;
+	RsvgState *state;
 	char buf [G_ASCII_DTOSTR_BUF_SIZE];
 	gboolean got_rx = FALSE, got_ry = FALSE;
 
+	/* skip over defs entries for now */
+	if (ctx->in_defs) return;
+
+	state = &ctx->state[ctx->n_state - 1];
+
 	if (atts != NULL)
 		{
 			for (i = 0; atts[i] != NULL; i += 2)
@@ -451,6 +474,8 @@ rsvg_start_rect (RsvgHandle *ctx, const xmlChar **atts)
 					}
 					else if (!strcmp ((char *)atts[i], "class"))
 						klazz = (const char *)atts[i + 1];
+					else if (!strcmp ((char *)atts[i], "id"))
+						id = (const char *)atts[i + 1];
 				}
 		}
 
@@ -460,32 +485,12 @@ rsvg_start_rect (RsvgHandle *ctx, const xmlChar **atts)
 	if (x < 0. || y < 0. || w < 0. || h < 0. || rx < 0. || ry < 0.)
 		return;
 	
-	rsvg_parse_style_attrs (ctx, "rect", klazz, atts);
+	rsvg_parse_style_attrs (ctx, "rect", klazz, id, atts);
 	
 	/* incrementing y by 1 properly draws borders. this is a HACK */
 	y += 1.;
 	
 	/* emulate a rect using a path */
-	/*
-	d = g_strdup_printf ("M %f %f "
-						 "H %f "
-						 "A %f,%f %f,%f %f %f,%f "
-						 "V %f "
-						 "A %f,%f %f,%f %f %f,%f "
-						 "H %f "
-						 "A %f,%f %f,%f %f %f,%f "
-						 "V %f "
-						 "A %f,%f %f,%f %f %f,%f",
-						 x + rx, y,
-						 x + w - rx,
-						 rx, ry, 0., 0., 1., x + w, y + ry,
-						 y + h - ry,
-						 rx, ry, 0., 0., 1., x + w - rx, y + h,
-						 x + rx,
-						 rx, ry, 0., 0., 1., x, y + h - ry,
-						 y + ry,
-						 rx, ry, 0., 0., 1., x + rx, y);
-	*/
 
 	d = g_string_new ("M ");
 	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), x + rx));
@@ -574,10 +579,15 @@ rsvg_start_circle (RsvgHandle *ctx, const xmlChar **atts)
 	int i;
 	double cx = 0, cy = 0, r = 0;
 	GString * d = NULL;
-	const char * klazz = NULL;
-	RsvgState *state = &ctx->state[ctx->n_state - 1];
+	const char * klazz = NULL, * id = NULL;
+	RsvgState *state;
 	char buf [G_ASCII_DTOSTR_BUF_SIZE];
 
+	/* skip over defs entries for now */
+	if (ctx->in_defs) return;
+
+	state = &ctx->state[ctx->n_state - 1];
+
 	if (atts != NULL)
 		{
 			for (i = 0; atts[i] != NULL; i += 2)
@@ -592,29 +602,17 @@ rsvg_start_circle (RsvgHandle *ctx, const xmlChar **atts)
 															  state->font_size);
 					else if (!strcmp ((char *)atts[i], "class"))
 						klazz = (const char *)atts[i + 1];
+					else if (!strcmp ((char *)atts[i], "id"))
+						id = (const char *)atts[i + 1];
 				}
 		}
 	
 	if (cx < 0. || cy < 0. || r <= 0.)
 		return;
 	
-	rsvg_parse_style_attrs (ctx, "circle", klazz, atts);
+	rsvg_parse_style_attrs (ctx, "circle", klazz, id, atts);
 	
 	/* approximate a circle using 4 bezier curves */
-	/* 
-	d = g_strdup_printf ("M %f %f "
-						 "C %f %f %f %f %f %f "
-						 "C %f %f %f %f %f %f "
-						 "C %f %f %f %f %f %f "
-						 "C %f %f %f %f %f %f "
-						 "Z",
-						 cx + r, cy,
-						 cx + r, cy + r * RSVG_ARC_MAGIC, cx + r * RSVG_ARC_MAGIC, cy + r, cx, cy + r,
-						 cx - r * RSVG_ARC_MAGIC, cy + r, cx - r, cy + r * RSVG_ARC_MAGIC, cx - r, cy,
-						 cx - r, cy - r * RSVG_ARC_MAGIC, cx - r * RSVG_ARC_MAGIC, cy - r, cx, cy - r,
-						 cx + r * RSVG_ARC_MAGIC, cy - r, cx + r, cy - r * RSVG_ARC_MAGIC, cx + r, cy
-						 );
-	*/
 
 	d = g_string_new ("M ");
 	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx+r));
@@ -685,10 +683,15 @@ rsvg_start_ellipse (RsvgHandle *ctx, const xmlChar **atts)
 	int i;
 	double cx = 0, cy = 0, rx = 0, ry = 0;
 	GString * d = NULL;
-	const char * klazz = NULL;
-	RsvgState *state = &ctx->state[ctx->n_state - 1];
+	const char * klazz = NULL, * id = NULL;
+	RsvgState *state;
 	char buf [G_ASCII_DTOSTR_BUF_SIZE];
 
+	/* skip over defs entries for now */
+	if (ctx->in_defs) return;
+
+	state = &ctx->state[ctx->n_state - 1];
+
 	if (atts != NULL)
 		{
 			for (i = 0; atts[i] != NULL; i += 2)
@@ -703,29 +706,18 @@ rsvg_start_ellipse (RsvgHandle *ctx, const xmlChar **atts)
 						ry = rsvg_css_parse_normalized_length ((char *)atts[i + 1], ctx->dpi, (gdouble)ctx->height, state->font_size);
 					else if (!strcmp ((char *)atts[i], "class"))
 						klazz = (const char *)atts[i + 1];
+					else if (!strcmp ((char *)atts[i], "id"))
+						id = (const char *)atts[i + 1];
 				}
 		}
 	
 	if (cx < 0. || cy < 0. || rx <= 0. || ry <= 0.)
 		return;
 	
-	rsvg_parse_style_attrs (ctx, "ellipse", klazz, atts);
+	rsvg_parse_style_attrs (ctx, "ellipse", klazz, id, atts);
 	
 	/* approximate an ellipse using 4 bezier curves */
-	/*
-	d = g_strdup_printf ("M %f %f "
-						 "C %f %f %f %f %f %f "
-						 "C %f %f %f %f %f %f "
-						 "C %f %f %f %f %f %f "
-						 "C %f %f %f %f %f %f "
-						 "Z",
-						 cx + rx, cy,
-						 cx + rx, cy - RSVG_ARC_MAGIC * ry, cx + RSVG_ARC_MAGIC * rx, cy - ry, cx, cy - ry,
-						 cx - RSVG_ARC_MAGIC * rx, cy - ry, cx - rx, cy - RSVG_ARC_MAGIC * ry, cx - rx, cy,
-						 cx - rx, cy + RSVG_ARC_MAGIC * ry, cx - RSVG_ARC_MAGIC * rx, cy + ry, cx, cy + ry,
-						 cx + RSVG_ARC_MAGIC * rx, cy + ry, cx + rx, cy + RSVG_ARC_MAGIC * ry, cx + rx, cy
-						 );
-	*/
+
 	d = g_string_new ("M ");
 	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx + rx));
 	g_string_append_c (d, ' ');
@@ -799,7 +791,7 @@ rsvg_start_image (RsvgHandle *ctx, const xmlChar **atts)
 	int i;
 	double x = 0., y = 0., w = -1., h = -1.;
 	const char * href = NULL;
-	const char * klazz = NULL;
+	const char * klazz = NULL, * id = NULL;
 	
 	GdkPixbuf *img;
 	GError *err = NULL;
@@ -808,7 +800,12 @@ rsvg_start_image (RsvgHandle *ctx, const xmlChar **atts)
 	guchar *rgb = NULL;
 	int dest_rowstride;
 	double tmp_affine[6];
-	RsvgState *state = &ctx->state[ctx->n_state - 1];
+	RsvgState *state;
+
+	/* skip over defs entries for now */
+	if (ctx->in_defs) return;
+
+	state = &ctx->state[ctx->n_state - 1];
 	
 	if (atts != NULL)
 		{
@@ -827,13 +824,15 @@ rsvg_start_image (RsvgHandle *ctx, const xmlChar **atts)
 						href = (const char *)atts[i + 1];
 					else if (!strcmp ((char *)atts[i], "class"))
 						klazz = (const char *)atts[i + 1];
+					else if (!strcmp ((char *)atts[i], "id"))
+						id = (const char *)atts[i + 1];
 				}
 		}
 	
 	if (!href || x < 0. || y < 0. || w <= 0. || h <= 0.)
 		return;
 	
-	rsvg_parse_style_attrs (ctx, "image", klazz, atts);
+	rsvg_parse_style_attrs (ctx, "image", klazz, id, atts);
 	
 	/* figure out if image is visible or not */
 	if (!state->visible)
